@@ -18,6 +18,7 @@ import navis
 import requests
 import textwrap
 
+import datetime as dt
 import cloudvolume as cv
 import numpy as np
 import pandas as pd
@@ -39,10 +40,84 @@ except ImportError:
 except BaseException:
     raise
 
-__all__ = ['fetch_edit_history', 'locs_to_segments',
+__all__ = ['fetch_edit_history', 'fetch_leaderboard', 'locs_to_segments',
            'locs_to_supervoxels', 'skid_to_id', 'update_ids']
 
 
+def fetch_leaderboard(days=7, by_day=False, progress=True, max_threads=4):
+    """Fetch leader board (# of edits).
+
+    Parameters
+    ----------
+    day :           int
+                    Number of days to go back.
+    by_day :        bool
+                    If True, will provide a day-by-day breakdown of # edits.
+    progress :      bool
+                    If True, show progress bar.
+    max_threads :   int
+                    Max number of parallel requests to server.
+
+    Returns
+    -------
+    pandas.DataFrame
+
+    Examples
+    --------
+    >>> from fafbseg import flywire
+    >>> # Fetch leaderboard
+    >>> edits = flywire.fetch_edit_history(720575940621039145)
+    >>> # Group by user
+    >>> edits.groupby('user_name').size()
+    user_name
+    Claire McKellar    47
+    Jay Gager           4
+    Sandeep Kumar       1
+    Sarah Morejohn      6
+    dtype: int64
+
+    """
+    assert isinstance(days, (int, np.int))
+    assert days >= 0
+
+    session = requests.Session()
+    if not by_day:
+        url = f'https://pyrdev.eyewire.org/flywire-leaderboard?days={days-1}'
+        resp = session.get(url, params=None)
+        resp.raise_for_status()
+        return pd.DataFrame.from_records(resp.json()['entries']).set_index('name')
+
+    future_session = FuturesSession(session=session, max_workers=max_threads)
+    futures = []
+    for i in range(0, days):
+        url = f'https://pyrdev.eyewire.org/flywire-leaderboard?days={i}'
+        futures.append(future_session.get(url, params=None))
+
+    # Get the responses
+    resp = [f.result() for f in tqdm(futures,
+                                     desc='Fetching',
+                                     disable=not progress or len(futures) == 1,
+                                     leave=False)]
+
+    df = None
+    for i, r in enumerate(resp):
+        date = dt.date.today() - dt.timedelta(days=i)
+        r.raise_for_status()
+        this_df = pd.DataFrame.from_records(r.json()['entries']).set_index('name')
+        this_df.columns = [date]
+        if isinstance(df, type(None)):
+            df = this_df
+        else:
+            df = pd.merge(df, this_df, how='outer', left_index=True, right_index=True)
+        df = df.fillna(0).astype(int)
+
+        if df.shape[1] > 1:
+            df.iloc[:, -1] -= df.iloc[:, :-1].sum(axis=1)
+
+    df = df.iloc[:, ::-1]
+    return df.loc[df.sum(axis=1).sort_values(ascending=False).index]
+
+
 def fetch_edit_history(x, dataset='production', progress=True, max_threads=4):
     """Fetch edit history for given neuron(s).
 
@@ -103,11 +178,17 @@ def fetch_edit_history(x, dataset='production', progress=True, max_threads=4):
         r.raise_for_status()
         this_df = pd.DataFrame(r.json())
         this_df['segment'] = i
-        if not this_df.empty:
-            df.append(this_df)
+        df.append(this_df)
 
-    df = pd.concat(df, axis=0, sort=True)
-    df['timestamp'] = pd.to_datetime(df.timestamp, unit='ms')
+    # Concat if any edits at all
+    if any([not f.empty for f in df]):
+        # Drop neurons without edits
+        df = [f for f in df if not f.empty]
+        df = pd.concat(df, axis=0, sort=True)
+        df['timestamp'] = pd.to_datetime(df.timestamp, unit='ms')
+    else:
+        # Return the first empty data frame
+        df = df[0]
 
     return df
 
@@ -119,8 +200,10 @@ def locs_to_supervoxels(locs, mip=2, coordinates='pixel'):
 
     Parameters
     ----------
-    locs :          list-like
-                    Array of x/y/z coordinates.
+    locs :          list-like | pandas.DataFrame
+                    Array of x/y/z coordinates. If DataFrame must contain
+                    'x', 'y', 'z' or 'fw.x', 'fw.y', 'fw.z' columns. If both
+                    present, 'fw.' columns take precedence)!
     mip :           int [2-8]
                     Scale to query. Lower mip = more precise but slower;
                     higher mip = faster but less precise (small supervoxels
@@ -144,6 +227,19 @@ def locs_to_supervoxels(locs, mip=2, coordinates='pixel'):
     array([79801454835332154, 79731086091150780], dtype=uint64)
 
     """
+    if isinstance(locs, pd.DataFrame):
+        if np.all(np.isin(['fw.x', 'fw.y', 'fw.z'], locs.columns)):
+            locs = locs[['fw.x', 'fw.y', 'fw.z']].values
+        elif np.all(np.isin(['x', 'y', 'z'], locs.columns)):
+            locs = locs[['x', 'y', 'z']].values
+        else:
+            raise ValueError('`locs` as pandas.DataFrame must have either [fw.x'
+                             ', fw.y, fw.z] or [x, y, z] columns.')
+
+        # Make sure we are working with numbers
+        if not np.issubdtype(locs.dtype, np.number):
+            locs = locs.astype(np.float64)
+
     return utils.query_spine(locs,
                              dataset='flywire_190410',
                              query='query',
@@ -157,8 +253,10 @@ def locs_to_segments(locs, root_ids=True, dataset='production',
 
     Parameters
     ----------
-    locs :          list-like
-                    Array of x/y/z coordinates.
+    locs :          list-like | pandas.DataFrame
+                    Array of x/y/z coordinates. If DataFrame must contain
+                    'x', 'y', 'z' or 'fw.x', 'fw.y', 'fw.z' columns. If both
+                    present, 'fw.' columns take precedence)!
     root_ids :      bool
                     If True, will return root IDs. If False, will return supervoxel
                     IDs.
@@ -185,7 +283,7 @@ def locs_to_segments(locs, root_ids=True, dataset='production',
     array([720575940621039145, 720575940621039145])
 
     """
-    svoxels = locs_to_supervoxels(locs)
+    svoxels = locs_to_supervoxels(locs, coordinates=coordinates)
 
     if not root_ids:
         return svoxels
@@ -199,8 +297,7 @@ def locs_to_segments(locs, root_ids=True, dataset='production',
     return roots
 
 
-def skid_to_id(skid,
-               sample=.5,
+def skid_to_id(x,
                dataset='production',
                progress=True, **kwargs):
     """Find the flywire ID(s) corresponding to given CATMAID skeleton ID(s).
@@ -213,9 +310,9 @@ def skid_to_id(skid,
 
     Parameters
     ----------
-    id :            int | list-like | str | TreeNeuron/List
-                    Anything that's not a TreeNeuron/List be passed directly
-                    to ``pymaid.get_neuron``.
+    x :             int | list-like | str | TreeNeuron/List
+                    Anything that's not a TreeNeuron/List will be passed
+                    directly to ``pymaid.get_neuron``.
     dataset :       str | CloudVolume
                     Against which flywire dataset to query::
                         - "production" (current production dataset, fly_v31)
@@ -235,42 +332,26 @@ def skid_to_id(skid,
     """
     vol = parse_volume(dataset, **kwargs)
 
-    if not isinstance(skid, (navis.TreeNeuron, navis.NeuronList)):
-        skid = pymaid.get_neuron(skid)
+    if not isinstance(x, (navis.TreeNeuron, navis.NeuronList)):
+        x = pymaid.get_neuron(x)
 
-    if isinstance(skid, navis.TreeNeuron):
-        nodes = skid.nodes[['x', 'y', 'z']]
-        nodes['skeleton_id'] = skid.id
-    elif isinstance(skid, navis.NeuronList):
-        nodes = skid.nodes[['x', 'y', 'z']]
+    if isinstance(x, navis.TreeNeuron):
+        nodes = x.nodes[['x', 'y', 'z']].copy()
+        nodes['skeleton_id'] = x.id
+    elif isinstance(x, navis.NeuronList):
+        nodes = x.nodes[['x', 'y', 'z']].copy()
     else:
-        raise TypeError(f'Unable to data of type "{type(skid)}"')
+        raise TypeError(f'Unable to data of type "{type(x)}"')
 
     # XForm coordinates from FAFB14 to FAFB14.1
-    nodes[['xf', 'yf', 'zf']] = xform.fafb14_to_flywire(nodes[['x', 'y', 'z']].values,
-                                                        coordinates='nanometers')
+    xformed = xform.fafb14_to_flywire(nodes[['x', 'y', 'z']].values,
+                                      coordinates='nm')
 
     # Get the root IDs for each of these locations
-    nodes['root_id'] = locs_to_segments(nodes[['xf', 'yf', 'zf']].values,
-                                        coordinates='nm')
+    roots = locs_to_segments(xformed, coordinates='nm')
 
-    # Get supervoxel ids - we need to use mip=0 because otherwise small neurons might
-    # not have any (visible) supervoxels
-    svoxels = vol.get_leaves(id, bbox=vol.meta.bounds(0), mip=0)
-
-    # Shuffle voxels
-    np.random.shuffle(svoxels)
-
-    # Generate sample
-    if isinstance(sample, type(None)):
-        smpl = svoxels
-    elif sample >= 1:
-        smpl = svoxels[: sample]
-    else:
-        smpl = svoxels[: int(len(svoxels) * sample)]
-
-    # Fetch up-to-date root IDs for the sampled supervoxels
-    roots = vol.get_roots(smpl)
+    # Drop zeros
+    roots = roots[roots != 0]
 
     # Find unique Ids and count them
     unique, counts = np.unique(roots, return_counts=True)
@@ -281,14 +362,14 @@ def skid_to_id(skid,
     # New Id is the most frequent ID
     new_id = unique[sort_ix[-1]]
 
-    # Confidence is the difference between the top and the 2bd most frequent ID
+    # Confidence is the difference between the top and the 2nd most frequent ID
     if len(unique) > 1:
-        conf = round((counts[sort_ix[-1]] - counts[sort_ix[-2]]) / sum(counts),
-                     2)
+        diff_1st_2nd = counts[sort_ix[-1]] - counts[sort_ix[-2]]
+        conf = round(diff_1st_2nd / roots.shape[0], 2)
     else:
         conf = 1
 
-    return pd.DataFrame([[id, new_id, conf, id != new_id]],
+    return pd.DataFrame([[x.id, new_id, conf, x.id != new_id]],
                         columns=['old_id', 'new_id', 'confidence', 'changed'])
 
 
@@ -312,9 +393,9 @@ def update_ids(id,
                     Number (>= 1) or fraction (< 1) of super voxels to sample
                     to guess the most recent version.
     dataset :       str | CloudVolume
-                    Against which flywire dataset to query::
-                        - "production" (current production dataset, fly_v31)
-                        - "sandbox" (i.e. fly_v26)
+                    Against which flywire dataset to query:
+                      - "production" (current production dataset, fly_v31)
+                      - "sandbox" (i.e. fly_v26)
     progress :      bool
                     If True, shows progress bar.
 
@@ -373,7 +454,7 @@ def update_ids(id,
     # New Id is the most frequent ID
     new_id = unique[sort_ix[-1]]
 
-    # Confidence is the difference between the top and the 2bd most frequent ID
+    # Confidence is the difference between the top and the 2nd most frequent ID
     if len(unique) > 1:
         conf = round((counts[sort_ix[-1]] - counts[sort_ix[-2]]) / sum(counts),
                      2)
