@@ -17,8 +17,12 @@ import navis
 import json
 import os
 
+from pathlib import Path
+from importlib import reload
+
 import cloudvolume as cv
 
+
 __all__ = ['set_chunkedgraph_secret']
 
 FLYWIRE_DATASETS = {'production': 'fly_v31',
@@ -47,11 +51,25 @@ def set_chunkedgraph_secret(token, filepath=None):
     elif not filepath.endswith('.json'):
         filepath = f'{filepath}.json'
 
-    filepath = os.path.expanduser(filepath)
+    filepath = Path(filepath).expanduser()
 
-    with open(filepath, 'w') as f:
+    # Make sure this file (and the path!) actually exist
+    if not filepath.exists():
+        if not filepath.parent.exists():
+            filepath.parent.mkdir(parents=True)
+        filepath.touch()
+
+    with open(filepath, 'w+') as f:
         json.dump({'token': token}, f)
 
+    # We need to reload cloudvolume for changes to take effect
+    reload(cv.secrets)
+    reload(cv)
+
+    # Should also reset the volume after setting the secret
+    global fw_vol
+    fw_vol = None
+
     print("Token succesfully stored in ", filepath)
 
 
@@ -62,7 +80,7 @@ def parse_volume(vol, **kwargs):
         if not isinstance(vol, str):
             raise ValueError(f'Unable to initialize CloudVolume from "{type(vol)}"')
 
-        if not navis.is_url(vol):
+        if not navis.utils.is_url(vol):
             # We are assuming this is the dataset
             # Map "production" and "sandbox" with to their correct designations
             vol = FLYWIRE_DATASETS.get(vol, vol)
@@ -84,6 +102,13 @@ def parse_volume(vol, **kwargs):
                             progress=False)
             defaults.update(kwargs)
 
+            # Check if chunkedgraph secret exists
+            secret = os.path.expanduser('~/.cloudvolume/secrets/chunkedgraph-secret.json')
+            if not os.path.isfile(secret):
+                # If not secrets but environment variable use this
+                if 'CHUNKEDGRAPH_SECRET' in os.environ and 'secrets' not in defaults:
+                    defaults['secrets'] = {'token': os.environ['CHUNKEDGRAPH_SECRET']}
+
             fw_vol = cv.CloudVolume(vol, **defaults)
             fw_vol.path = vol
     else:
